/**
 * 
 */
package net.java.amateras.uml.sequence.action;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;

import net.java.amateras.uml.ClassDiagramSerializer;
import net.java.amateras.uml.UMLPlugin;
import net.java.amateras.uml.action.AbstractUMLEditorAction;
import net.java.amateras.uml.model.AbstractUMLEntityModel;
import net.java.amateras.uml.model.AttributeModel;
import net.java.amateras.uml.model.ClassModel;
import net.java.amateras.uml.model.EntityModel;
import net.java.amateras.uml.model.InterfaceModel;
import net.java.amateras.uml.model.OperationModel;
import net.java.amateras.uml.model.RootModel;
import net.java.amateras.uml.sequence.ClassModelImageResolver;
import net.java.amateras.uml.sequence.editpart.InstanceEditPart;
import net.java.amateras.uml.sequence.model.InstanceModel;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.dialogs.ISelectionStatusValidator;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.model.WorkbenchLabelProvider;

/**
 * Load RootModel to sequence diagram editor.
 * 
 * @author Takahiro Shida.
 * 
 */
public class ImportClassModelAction extends AbstractUMLEditorAction {

	private CommandStack stack;

	private InstanceModel model;

	public ImportClassModelAction(CommandStack stack, GraphicalViewer viewer) {
		super(
				UMLPlugin.getDefault().getResourceString(
						"menu.importClass"), viewer);
		this.stack = stack;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.java.amateras.uml.action.AbstractUMLEditorAction#update(org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void update(IStructuredSelection sel) {
		Object element = sel.getFirstElement();
		if (element instanceof InstanceEditPart) {
			InstanceEditPart editPart = (InstanceEditPart) element;
			model = (InstanceModel) editPart.getModel();
			setEnabled(true);
		} else {
			setEnabled(false);
		}
	}

	public void run() {
		IFile file = selectClassDiagram();
		if (file == null) {
			return;
		}
		final AbstractUMLEntityModel entityModel = selectClassModel(file);
		if (entityModel == null) {
			return;
		}
		stack.execute(new Command() {
		
			private EntityModel old;
			public void execute() {
				old = model.getType();
				model.setType((EntityModel) entityModel);
			}
			
			public void undo() {
				model.setType(old);
			}
		});
		super.run();
	}
	
	private IFile selectClassDiagram() {
		ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(
				PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
				new WorkbenchLabelProvider(), new WorkbenchContentProvider());
		dialog.setInput(ResourcesPlugin.getWorkspace());
		dialog.setAllowMultiple(false);
		dialog.setTitle(UMLPlugin.getDefault().getResourceString("fileSelectionDialog.title"));
		dialog.setValidator(new ISelectionStatusValidator() {

			public IStatus validate(Object[] selection) {
				for (int i = 0; i < selection.length; i++) {
					Object object = selection[i];
					if (object instanceof IFile) {
						IFile file = (IFile) object;
						if (file.getName().endsWith(".cld")) {
							return Status.OK_STATUS;
						}
						return new Status(IStatus.ERROR, UMLPlugin.PLUGIN_ID, 0,"",null);
					}
				}
				return new Status(IStatus.ERROR, UMLPlugin.PLUGIN_ID, 0,"",null);
			}
			
		});
		if (dialog.open() == Dialog.OK) {
			IFile file = (IFile) dialog.getFirstResult();
			return file;
		}
		return null;
	}
	
	private AbstractUMLEntityModel selectClassModel(IFile file) {
		try {
			RootModel rootModel = ClassDiagramSerializer.deserialize(new FileInputStream(file.getLocation().toFile()));
			ElementTreeSelectionDialog dialog = new ElementTreeSelectionDialog(
					PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
					new ClassDiagramLabelProvider(),
					new ClassDiagramContentProvider()
					);
			dialog.setInput(rootModel);
			dialog.setAllowMultiple(false);
			dialog.setTitle(UMLPlugin.getDefault().getResourceString("modelSelectionDialog.title"));
			dialog.setValidator(new ISelectionStatusValidator() {

				public IStatus validate(Object[] selection) {
					for (int i = 0; i < selection.length; i++) {
						Object object = selection[i];
						if (object instanceof AbstractUMLEntityModel) {
							return Status.OK_STATUS;
						}
					}
					return new Status(IStatus.ERROR, UMLPlugin.PLUGIN_ID, 0,"",null);
				}
				
			});
			if (dialog.open() == Dialog.OK) {
				return (AbstractUMLEntityModel) dialog.getFirstResult();
			}
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	private class ClassDiagramContentProvider implements ITreeContentProvider {

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof RootModel) {
				RootModel model = (RootModel) parentElement;
				return model.getChildren().toArray();
			} else if (parentElement instanceof AbstractUMLEntityModel) {
				AbstractUMLEntityModel model = (AbstractUMLEntityModel) parentElement;
				return model.getChildren().toArray();
			} 
			return null;
		}

		public Object getParent(Object element) {
			// TODO Auto-generated method stub
			return null;
		}

		public boolean hasChildren(Object element) {
			return element instanceof RootModel || element instanceof ClassModel;
		}

		public Object[] getElements(Object inputElement) {
			return getChildren(inputElement);
		}

		public void dispose() {
			
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
			
		}
		
	}
	
	private class ClassDiagramLabelProvider extends LabelProvider {
		
		public Image getImage(Object element) {
			if (element instanceof InterfaceModel) {
				return UMLPlugin.getImageDescriptor("icons/interface.gif").createImage();
			} else if (element instanceof ClassModel) {
				return UMLPlugin.getImageDescriptor("icons/class.gif").createImage();
			} else if (element instanceof AttributeModel) {
				return ClassModelImageResolver.getAttributeImage((AttributeModel) element);
			} else if (element instanceof OperationModel) {
				return ClassModelImageResolver.getOperationImage((OperationModel) element);
			}
			return super.getImage(element);
		}
		
		public String getText(Object element) {
			if (element instanceof EntityModel) {
				return ((EntityModel) element).getName();
			}
			return super.getText(element);
		}
	}
}
